/*
Package policies provides information and interaction with the policy API
resource in the Rackspace Auto Scale service.

Auto Scale uses policies to define when and how scaling activity will take
place.  Scaling policies specify how to modify the scaling group and its
behavior.  You can specify multiple policies to manage a scaling group.
*/
package policies
