/*
Package webhooks provides information and interaction with the webhook API resource
in the Rackspace Auto Scale service.

Auto Scale uses webhooks to initiate scaling events.  Webhooks are associated
with scaling policies and provide capability URLs which can be accessed
anonymously to trigger execution of those policies.  The Auto Scale webhook
architecture allows Auto Scale to be integrated with other systems, for example,
monitoring systems.
*/
package webhooks
